DROP VIEW IF EXISTS "sa_hosts_from_roles" CASCADE;
DROP VIEW IF EXISTS "sa_hosts_all_users" CASCADE;
DROP VIEW IF EXISTS "sa_hosts_from_users_groups" CASCADE;
DROP VIEW IF EXISTS "sa_hosts_from_he" CASCADE;
--DROP VIEW IF EXISTS "sa_hosts" CASCADE;
DROP VIEW IF EXISTS "role_mappings" CASCADE;
-- Makes this show only the hosts for which a given user has access to (is in a role)
--select hosts.hostname, r.role from types_roles tr, roles r, types t, hosts_types ht, hosts WHERE r.id = tr.role_id AND tr.type_id = t.id and ht.type_id = tr.type_id AND ht.host_id = hosts.id order by hostname;

-- See access to hosts from roles. Considering the roles, what gives a user access.
CREATE VIEW "sa_hosts_from_roles" AS
  SELECT
    u.id              AS user_id,
    h.hostname,
    roles.role        AS reason_label,
    text 'role'       AS why,
    CAST(NULL as INT) AS group_group_id
  FROM
    users           u,
    roles_entities re,
    types_roles    tr,
    hosts_types    ht,
    hosts           h,
    roles
  WHERE
    u.id          = re.entity_record_id AND
    re.entity_id  = 1           AND
    re.role_id    = tr.role_id  AND
    re.role_id    = roles.id    AND
    tr.type_id    = ht.type_id  AND
    ht.host_id    = h.id
  ;

-- See what groups a user is in on hosts. Kinda, this one is kinda ugly.
CREATE VIEW "sa_hosts_from_users_groups" AS
  SELECT
    u.id        AS user_id,
    h.hostname  AS hostname,
    r.role      AS reason_label,
    text 'users_groups' as why,
    u_g.group_id AS group_group_id
  FROM
    users           u
    INNER JOIN users_groups       u_g ON u.id = u_g.user_id
    FULL OUTER JOIN roles_entities re ON u_g.id = re.entity_record_id
    LEFT OUTER JOIN roles           r on re.role_id = r.id
    INNER JOIN      entities        e ON re.entity_id = e.id
    FULL OUTER JOIN types_roles    tr ON re.role_id   = tr.role_id
    FULL OUTER JOIN hosts_types    ht ON tr.type_id   = ht.type_id
    FULL OUTER JOIN hosts           h ON ht.host_id   = h.id
  WHERE
    e.entity = 'users_groups';

-- Don't See access to Hosts from a user's primary group.
--  Don't bother. Most primary groups are mapped to Role ANY.
-- {{{
CREATE VIEW "sa_hosts_from_he" AS
  SELECT
    u.id                  AS user_id,
    h.hostname            AS hostname,
    e.entity              AS reason_label,
    text 'hosts_entities' AS why,
    NULL AS group_group_id
  FROM
    hosts h,
    hosts_entities he INNER JOIN entities e ON he.entity_id = e.id,
    users u
  WHERE
    u.id = he.entity_record_id AND
    e.entity = 'users'         AND
    he.host_id = h.id

-- This part of the UNION doesn't seem to be useful.
UNION

-- users_groups
  SELECT
    u.id,
    h.hostname,
    g.groupname AS reason_label,
    e.entity    AS why,
    CAST(NULL AS int) AS group_group_id
  FROM
    users u
    INNER JOIN users_groups ug ON u.id = ug.user_id
    FULL OUTER JOIN hosts_entities he ON ug.id = he.entity_record_id
    INNER JOIN entities e ON he.entity_id = e.id
    INNER JOIN hosts    h ON he.host_id   = h.id
    INNER JOIN groups   g ON ug.group_id  = g.id
  WHERE
    e.entity = 'users_groups';

-- }}}

-- see hosts *all users* have access to
CREATE VIEW "sa_hosts_all_users" AS
  SELECT
    CAST(NULL AS int)  AS user_id,
    hosts.hostname     AS hostname,
    types.type         AS reason_label,
    text 'hosts_types' AS why,
    CAST(NULL AS int)  AS group_group_id
    FROM hosts
      INNER JOIN hosts_types ON hosts.id = hosts_types.host_id
      INNER JOIN types       ON hosts_types.type_id = types.id
    WHERE
      types.type = 'ALL:users'
  ;


-- See what's applied to roles
CREATE VIEW "role_mappings" AS
SELECT r_e.role_id, entities.entity, users.id, users.username as primary, NULL as secondary FROM roles_entities r_e
  INNER JOIN entities ON r_e.entity_id = entities.id
  INNER JOIN users    ON r_e.entity_record_id = users.id
  WHERE entities.entity = 'users' UNION
SELECT r_e.role_id, entities.entity, groups.id, groups.groupname, NULL as secondary FROM roles_entities r_e
  INNER JOIN entities ON r_e.entity_id = entities.id
  INNER JOIN groups ON r_e.entity_record_id = groups.id
  WHERE entities.entity = 'groups' UNION
SELECT r_e.role_id, entities.entity, users.id, users.username, groups.groupname FROM roles_entities r_e
  INNER JOIN entities         ON r_e.entity_id = entities.id
  INNER JOIN users_groups u_g ON r_e.entity_record_id = u_g.id
  INNER JOIN users            ON u_g.user_id = users.id
  INNER JOIN groups           ON u_g.group_id = groups.id
  WHERE entities.entity = 'users_groups';

SELECT h.id as host_id, h.hostname, h_e.entity_id, h_e.entity_record_id, u.username FROM hosts_entities h_e
  INNER JOIN hosts h    ON h_e.host_id          = h.id
  INNER JOIN entities e ON h_e.entity_id        = e.id
  INNER JOIN users u    ON h_e.entity_record_id = u.id
  WHERE entity = 'users';


-- Who has role ANY
SELECT * FROM users u
  INNER JOIN roles_entities r_e ON u.group_id = r_e.entity_record_id
  INNER JOIN entities e         ON r_e.entity_id = e.id
  INNER JOIN roles r            ON r_e.role_id = r.id
  WHERE e.entity = 'groups' AND r.role = 'ANY'
  ;
-- SELECT h.hostname as hostname FROM types_roles tr, hosts_types ht, hosts h WHERE
--    tr.type_id    = ht.type_id  AND
--    ht.host_id    = h.id
--    ;

--SELECT * FROM users u
--  INNER JOIN users_groups u_g   ON u.id = u_g.user_id
--  INNER JOIN groups g           ON u_g.group_id = g.id
--  INNER JOIN roles_entities r_e ON u_g.id = r_e.entity_record_id
--  INNER JOIN entities e         ON r_e.entity_id = e.id
--  INNER JOIN types_roles t_r    ON r_e.role_id = t_r.role_id
--  INNER JOIN hosts_types h_t    ON t_r.role_id = h_t.type_id
--  INNER JOIN hosts h            ON h_t.host_id = h.id
--  WHERE e.entity = 'users_groups'
--  ;

-- second page
--SELECT * FROM users u
--  INNER JOIN roles_entities r_e ON u.group_id = r_e.entity_record_id
--  INNER JOIN entities e         ON r_e.entity_id = e.id
--  INNER JOIN types_roles t_r    ON r_e.role_id = t_r.role_id
--  INNER JOIN hosts_types h_t    ON t_r.role_id = h_t.type_id
--  INNER JOIN hosts h            ON h_t.host_id = h.id
--  WHERE e.entity = 'groups'
--  ;

-- one'ze users on a host via user
--SELECT * from hosts h, hosts_entities he, users u WHERE u.id = he.entity_record_id AND he.entity_id = 1 AND he.host_id = h.id;



--SELECT r_e.* FROM roles_entities r_e INNER JOIN entities ON r_e.entity_id = entities.id
--  WHERE entities.entity = 'user_shell_override';


-- In Schema
SELECT "u".*,
        COALESCE("u"."shell", "m_d__default_user_shell"."value") as "shell_used"
    FROM
        "users"     "u",
        "misc_data" "m_d__default_user_shell"
    WHERE
        "m_d__default_user_shell"."key" = 'default_user_shell';

CREATE VIEW "sa_hosts" AS
  SELECT * FROM (
    SELECT * FROM sa_hosts_from_users_groups
  UNION
    SELECT * FROM sa_hosts_from_roles
  UNION
    SELECT * FROM sa_hosts_all_users
  UNION
    SELECT * FROM sa_hosts_from_he) AS data LEFT OUTER JOIN users on data.user_id = users.id
    ;
